@@ -9,8 +9,7 @@ env \
   cmake -DCMAKE_INSTALL_PREFIX="$REPOSITORYDIR" -DDEPENDENCY_SEARCH_PREFIX="$REPOSITORYDIR" \
   -DCMAKE_OSX_SYSROOT="macosx${SDKVERSION}" -DCMAKE_OSX_DEPLOYMENT_TARGET="$DEPLOY_TARGET" \
   -DWITH_OPENEXR=1 \
-  -DFFTW3F_LIBRARY="$REPOSITORYDIR/lib" -DFFTW3F_INCLUDE_DIR="$REPOSITORYDIR/include" || fail "configure step"
+  -DFFTW3F_LIBRARY="$REPOSITORYDIR/lib" -DFFTW3F_INCLUDE_DIR="$REPOSITORYDIR/include" . || fail "configure step"
 
-make clean || fail "make clean step"
 make $MAKEARGS || fail "make step"
 make install || fail "make install step"
\ No newline at end of file
