@@ -30,18 +30,18 @@
 #include "base_wx/wxPlatform.h"
 #include "panoinc.h"
 #include "hugin/huginApp.h"
-#if wxCHECK_VERSION(3,0,0)
 #include <wx/utils.h>
-#else
-#include <wx/version.h>
-#endif
 extern "C"
 {
 #include "pano13/queryfeature.h"
 }
-#if !(defined HAVE_STD_FILESYSTEM && defined HAVE_CXX11)
+#if !(defined HAVE_STD_FILESYSTEM)
 #include "boost/version.hpp"
 #endif
+#ifdef _WIN32
+// workaround for a conflict between exiv2 and wxWidgets/CMake built
+#define HAVE_PID_T 1
+#endif
 #include "exiv2/exiv2.hpp"
 #include "lensdb/LensDB.h"
 #include "sqlite3.h"
@@ -98,7 +98,7 @@ AboutDialog::AboutDialog(wxWindow *parent)
 	
 	// License
 	textCtrl = XRCCTRL(*this, "license_txt", wxTextCtrl);
-    strFile = huginApp::Get()->GetXRCPath() + wxT("data/COPYING");
+    strFile = huginApp::Get()->GetXRCPath() + wxT("data/COPYING.txt");
 #ifndef _WIN32
 	textCtrl->SetFont(font);
 #endif
@@ -148,7 +148,7 @@ void AboutDialog::GetSystemInformation(wxFont *font)
     text=text+wxT("\n")+wxString::Format(_("Architecture: %s"),is64.c_str());
     // wxGetFreeMemory returns a wxMemorySize, which is undocumented.
     // However, we know -1 is returned on failure, so it must be signed.
-    text=text+wxT("\n")+wxString::Format(_("Free memory: %lld kiB"),(long long) wxGetFreeMemory().GetValue()/1024);
+    text=text+wxT("\n")+wxString::Format(_("Free memory: %lld kiB"), wxGetFreeMemory().GetValue()/1024ll);
 #ifdef _WIN32
     UINT cp=GetACP();
     text=text+wxT("\n")+wxString::Format(_("Active Codepage: %u"),cp); 
@@ -188,31 +188,18 @@ void AboutDialog::GetSystemInformation(wxFont *font)
     text=text+wxT("\n")+wxString::Format(_("Path to data: %s"),huginApp::Get()->GetDataPath().c_str());
     HuginBase::LensDB::LensDB& lensDB=HuginBase::LensDB::LensDB::GetSingleton();
     text = text + wxT("\n") + wxString::Format(_("Hugins camera and lens database: %s"), wxString(lensDB.GetDBFilename().c_str(), wxConvLocal).c_str());
-#ifdef USE_CXX11_THREAD
     text = text + wxT("\n") + _("Multi-threading using C++11 std::thread and OpenMP");
-#else
-    text = text + wxT("\n") + _("Multi-threading using boost::thread and OpenMP");
-#endif
     if (huginApp::Get()->HasMonitorProfile())
     {
         text = text + wxT("\n") + wxString::Format(_("Monitor profile: %s"), huginApp::Get()->GetMonitorProfileName().c_str());
     }
     text=text+wxT("\n\n")+_("Libraries");
-#if wxCHECK_VERSION(3,0,0)
     wxVersionInfo info = wxGetLibraryVersionInfo();
     text = text + wxT("\nwxWidgets: ") + info.GetVersionString();
     if(info.HasDescription())
     {
         text=text+wxT("\n")+info.GetDescription();
     };
-#else
-    text=text+wxT("\n")+wxString::Format(wxT("wxWidgets: %i.%i.%i.%i"),
-                                            wxMAJOR_VERSION,
-                                            wxMINOR_VERSION,
-                                            wxRELEASE_NUMBER,
-                                            wxSUBRELEASE_NUMBER
-                                            );
-#endif
     {
         char panoVersion[255];
         if (queryFeatureString(PTVERSION_NAME_FILEVERSION, panoVersion, sizeof(panoVersion) / sizeof(panoVersion[0])))
@@ -220,10 +207,10 @@ void AboutDialog::GetSystemInformation(wxFont *font)
             text = text + wxT("\nlibpano13: ") + wxString(panoVersion, wxConvLocal);
         };
     }
-#if !(defined HAVE_STD_FILESYSTEM && defined HAVE_CXX11)
+#if !(defined HAVE_STD_FILESYSTEM)
     text=text+wxT("\n")+wxString::Format(wxT("Boost: %i.%i.%i"),BOOST_VERSION / 100000, BOOST_VERSION / 100 % 1000, BOOST_VERSION % 100);
 #endif
-    text = text + wxT("\n") + wxT("Exiv2: ") + wxString(Exiv2::version(), wxConvLocal);
+    text = text + wxT("\n") + wxT("Exiv2: ") + wxString(Exiv2::versionString().c_str(), wxConvLocal);
     text = text + wxT("\n") + wxT("SQLite3: ") + wxString(sqlite3_libversion(), wxConvLocal);
     text = text + wxT("\n") + wxString::Format(wxT("Vigra: %s"), wxString(VIGRA_VERSION, wxConvLocal).c_str());
     text = text + wxT("\n") + wxString::Format(wxT("LittleCMS2: %i.%i"), LCMS_VERSION / 1000, LCMS_VERSION / 10 % 100);
