@@ -25,7 +25,7 @@
  */
 
 // standard wx include
-#include <config.h>
+#include "hugin_config.h"
 #include "panoinc_WX.h"
 
 // standard hugin include
@@ -35,11 +35,7 @@
 #include <vigra/inspectimage.hxx>
 #include <vigra/transformimage.hxx>
 #include <vigra/basicimageview.hxx>
-#ifdef HAVE_CXX11
 #include <functional>  // std::bind 
-#else
-#include <boost/bind.hpp>
-#endif
 
 #include "hugin/config_defaults.h"
 #include "hugin/CPImageCtrl.h"
@@ -51,13 +47,6 @@
 
 #include "vigra_ext/ImageTransforms.h"
 
-#if !wxCHECK_VERSION(3,0,0)
-#define wxPENSTYLE_SOLID wxSOLID
-#define wxPENSTYLE_DOT wxDOT
-#define wxBRUSHSTYLE_TRANSPARENT wxTRANSPARENT
-#define wxBRUSHSTYLE_SOLID wxSOLID
-#endif
-
 // definition of the control point event
 
 IMPLEMENT_DYNAMIC_CLASS( CPEvent, wxEvent )
@@ -74,6 +63,13 @@ CPEvent::CPEvent( )
     mode = NONE;
 }
 
+CPEvent::CPEvent(wxWindow * win, CPEventMode evt_mode)
+{
+    SetEventType(EVT_CPEVENT);
+    SetEventObject(win);
+    mode = evt_mode;
+}
+
 CPEvent::CPEvent(wxWindow* win, hugin_utils::FDiff2D & p)
 {
     SetEventType( EVT_CPEVENT );
@@ -217,7 +213,7 @@ void DisplayedControlPoint::Draw(wxDC& dc, bool selected, bool newPoint)
     if(m_line)
     {
         box.setUpperLeft(vigra::Point2D(hugin_utils::roundi(std::min(m_cp.x1, m_cp.x2)) - l, hugin_utils::roundi(std::min(m_cp.y1, m_cp.y2)) - l));
-        box.setSize(hugin_utils::roundi(abs(m_cp.x1 - m_cp.x2) + 2.0*l), hugin_utils::roundi(abs(m_cp.y1 - m_cp.y2) + 2.0*l));
+        box.setSize(hugin_utils::roundi(std::abs(m_cp.x1 - m_cp.x2) + 2.0*l), hugin_utils::roundi(std::abs(m_cp.y1 - m_cp.y2) + 2.0*l));
     }
     else
     {
@@ -294,7 +290,7 @@ wxRect DisplayedControlPoint::DrawTextMag(wxDC& dc, wxPoint p, hugin_utils::FDif
     wxCoord tw, th;
     dc.GetTextExtent(m_label, &tw, &th);
 
-    if (drawMag)
+    if (drawMag && m_control->getScale() < 2)
     {
         wxBitmap magBitmap = m_control->generateMagBitmap(pointInput, p);
         // TODO: select position depending on visible part of canvas
@@ -778,6 +774,7 @@ void CPImageCtrl::OnDraw(wxDC & dc)
     case KNOWN_POINT_SELECTED:
         m_points[selectedPointNr].Draw(dc, true);
         break;
+    case SELECT_DELETE_REGION:
     case NO_SELECTION:
     case NO_IMAGE:
         break;
@@ -842,7 +839,7 @@ wxBitmap CPImageCtrl::generateMagBitmap(hugin_utils::FDiff2D point, wxPoint canv
 
     // apply the transform
     AppBase::DummyProgressDisplay progDisp;
-    vigra_ext::transformImageIntern(vigra::srcImageRange(*(m_img->image8)),
+    vigra_ext::transformImageIntern(vigra::srcImageRange(*m_img->get8BitImage()),
                          vigra::destImageRange(magImg),
                          vigra::destImage(maskImg),
                          transform,
@@ -924,11 +921,7 @@ void CPImageCtrl::setImage(const std::string & file, ImageRotation imgRot)
             // load the image in the background.
             m_imgRequest = ImageCache::getInstance().requestAsyncImage(imageFilename);
             m_imgRequest->ready.push_back(
-#ifdef HAVE_CXX11
                 std::bind(&CPImageCtrl::OnImageLoaded, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3)
-#else
-                boost::bind(&CPImageCtrl::OnImageLoaded, this, _1, _2, _3)
-#endif
                 );
             // With m_img.get() 0, everything will act as normal except drawing.
         }
@@ -1103,7 +1096,7 @@ void CPImageCtrl::showPosition(hugin_utils::FDiff2D point, bool warpPointer)
 CPImageCtrl::EditorState CPImageCtrl::isOccupied(wxPoint mousePos, const hugin_utils::FDiff2D &p, unsigned int & pointNr) const
 {
     // check if mouse is hovering over a label
-    if(m_points.size()>0)
+    if(!m_points.empty())
     {
         for(int i=m_points.size()-1; i>=0; i--)
         {
@@ -1148,7 +1141,6 @@ void CPImageCtrl::mouseMoveEvent(wxMouseEvent& mouse)
     hugin_utils::FDiff2D mpos(unScrolledMousePos.x, unScrolledMousePos.y);
     bool doUpdate = false;
     mpos = applyRotInv(invScale(mpos));
-    wxPoint mpos_ = applyRotInv(invScale(unScrolledMousePos));
     // if mouseclick is out of image, ignore
     if ((mpos.x >= m_realSize.GetWidth() || mpos.y >= m_realSize.GetHeight()) && editState!=SELECT_DELETE_REGION)
     {
@@ -1194,6 +1186,7 @@ void CPImageCtrl::mouseMoveEvent(wxMouseEvent& mouse)
             m_selectedPoint.UpdateControlPoint(mpos);
             doUpdate = true;
             break;
+        case SELECT_DELETE_REGION:
         case NO_IMAGE:
             break;
         }
@@ -1263,7 +1256,6 @@ void CPImageCtrl::mousePressLMBEvent(wxMouseEvent& mouse)
                            &unScrolledMousePos.x, & unScrolledMousePos.y);
     hugin_utils::FDiff2D mpos(unScrolledMousePos.x, unScrolledMousePos.y);
     mpos = applyRotInv(invScale(mpos));
-    wxPoint mpos_ = applyRotInv(invScale(unScrolledMousePos));
     DEBUG_DEBUG("mousePressEvent, pos:" << mpos.x
                 << ", " << mpos.y);
     // if mouseclick is out of image, ignore
@@ -1365,6 +1357,7 @@ void CPImageCtrl::mouseReleaseLMBEvent(wxMouseEvent& mouse)
             emit(e);
             break;
         }
+        case SELECT_DELETE_REGION:
         case NO_IMAGE:
             break;
 
@@ -1578,11 +1571,16 @@ void CPImageCtrl::OnKey(wxKeyEvent & e)
             update();
         }
 
-    } else if (e.m_keyCode == 'a') {
+    }
+    else if (e.m_keyCode == 'a') {
         DEBUG_DEBUG("adding point with a key, faking right click");
         // faking right mouse button with "a"
         // set right up event
-        CPEvent ev(this, CPEvent::RIGHT_CLICK, hugin_utils::FDiff2D(0,0));
+        CPEvent ev(this, CPEvent::RIGHT_CLICK, hugin_utils::FDiff2D(0, 0));
+        emit(ev);
+    }
+    else if (e.m_keyCode==WXK_ESCAPE) {
+        CPEvent ev(this, CPEvent::CANCELED);
         emit(ev);
     } else {
         // forward some keys...
@@ -1744,11 +1742,12 @@ void CPImageCtrl::ScrollDelta(const wxPoint & delta)
 
 const wxSize CPImageCtrl::GetBitmapSize() const
 {
-#if wxCHECK_VERSION(2,9,0)
     return bitmap.GetSize();
-#else
-    return wxSize(bitmap.GetWidth(), bitmap.GetHeight());
-#endif
+};
+
+HuginBase::ImageCache::ImageCacheRGB8Ptr CPImageCtrl::GetImg()
+{
+    return m_img->get8BitImage();
 };
 
 IMPLEMENT_DYNAMIC_CLASS(CPImageCtrl, wxScrolledWindow)
