@@ -15,22 +15,16 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
-#ifdef __APPLE__
 #include "panoinc_WX.h"
-#endif
-
-#include "hugin_utils/utils.h"
+#include "base_wx/wxutils.h"
 
 #include "panoinc.h"
-#include <config.h>
-#if !defined Hugin_shared || !defined _WINDOWS
-#define GLEW_STATIC
-#endif
+#include "hugin_config.h"
 #include <GL/glew.h>
 #include <base_wx/platform.h>
 #include <wx/settings.h>
@@ -47,6 +41,7 @@
 
 bool GLViewer::initialised_glew=false;
 ViewState * GLViewer::m_view_state = NULL;
+size_t GLViewer::m_view_state_observer = 0;
 
 BEGIN_EVENT_TABLE(GLViewer, wxGLCanvas)
     EVT_PAINT (GLViewer::RedrawE)
@@ -55,6 +50,7 @@ BEGIN_EVENT_TABLE(GLViewer, wxGLCanvas)
     // mouse motion
     EVT_MOTION (GLViewer::MouseMotion)
     // mouse entered or left the preview
+    EVT_ENTER_WINDOW(GLViewer::MouseEnter)
     EVT_LEAVE_WINDOW(GLViewer::MouseLeave)
     // mouse buttons
     EVT_MOUSEWHEEL(GLViewer::MouseWheel)
@@ -67,26 +63,16 @@ END_EVENT_TABLE()
 
 GLViewer::GLViewer(
             wxWindow* parent, 
-            PT::Panorama &pano, 
+            HuginBase::Panorama &pano, 
             int args[], 
             GLPreviewFrame *frame_in,
             wxGLContext * shared_context
             ) :
-#if defined __WXGTK__ || wxCHECK_VERSION(2,9,0)
-          wxGLCanvas(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize,
-                     0, wxT("GLPreviewCanvas"), args, wxNullPalette)
-#else
-          wxGLCanvas(parent,shared_context,wxID_ANY,wxDefaultPosition,
-                     wxDefaultSize,0,wxT("GLPreviewCanvas"),args,wxNullPalette)
-#endif
+          wxGLCanvas(parent, wxID_ANY, args, wxDefaultPosition, wxDefaultSize,
+                     0, wxT("GLPreviewCanvas"), wxNullPalette)
 {
-    /* The openGL display context doesn't seem to be created automatically on
-     * wxGTK, (wxMSW and wxMac 2.8 does implicit create wxGLContext,
-     * wxWidgets 2.9 requires to explicit create wxGLContext, 
-     * so I create a new context... */
-#if defined __WXGTK__ || wxCHECK_VERSION(2,9,0)
+    /* create OpenGL context... */
     m_glContext = new wxGLContext(this, shared_context);
-#endif
   
     m_renderer = 0;
     m_visualization_state = 0;
@@ -99,27 +85,28 @@ GLViewer::GLViewer(
     
     started_creation = false;
     redrawing = false;
+    m_toolsInitialized = false;
 
     active = true;
 }
 
 GLViewer::~GLViewer()
 {
-#if defined __WXGTK__ || wxCHECK_VERSION(2,9,0)
-    delete m_glContext;
-#endif
     if (m_renderer)
     {
       delete m_tool_helper;
       delete m_renderer;
+      delete m_visualization_state;
       // because m_view_state is a static member variable we need to check
       // if other class has already deleted it
-      if(m_view_state)
+      --m_view_state_observer;
+      if (m_view_state_observer == 0)
       {
         delete m_view_state;
         m_view_state=NULL;
       }
     }
+    delete m_glContext;
 }
 
 void GLViewer::SetUpContext()
@@ -127,11 +114,7 @@ void GLViewer::SetUpContext()
     // set the context
     DEBUG_INFO("Setting rendering context...");
     Show();
-#if defined __WXGTK__ || wxCHECK_VERSION(2,9,0)
     m_glContext->SetCurrent(*this);
-#else
-    SetCurrent();
-#endif
     DEBUG_INFO("...got a rendering context.");
     if (!started_creation)
     {
@@ -150,7 +133,7 @@ void GLViewer::SetUpContext()
                 DEBUG_ERROR("Error initialising GLEW: "
                         << glewGetErrorString(error_state) << ".");
                 frame->Close();
-                wxMessageBox(_("Error initializing GLEW\nFast preview window can not be opened."),_("Error"), wxOK | wxICON_ERROR,MainFrame::Get());
+                wxMessageBox(_("Error initializing GLEW\nFast preview window can not be opened."),_("Error"), wxOK | wxICON_ERROR);
                 return;
             }
         }
@@ -158,50 +141,90 @@ void GLViewer::SetUpContext()
         if (!(GLEW_VERSION_1_1 && GLEW_ARB_multitexture))
         {
             started_creation=false;
+            wxConfigBase::Get()->Write(wxT("DisableOpenGL"), 1l);
+            wxConfigBase::Get()->Flush();
             DEBUG_ERROR("Sorry, OpenGL 1.1 + GL_ARB_multitexture extension required.");
             frame->Close();
-            wxMessageBox(_("Sorry, the fast preview window requires a system which supports OpenGL version 1.1 with the GL_ARB_multitexture extension.\nThe fast preview cannot be opened."),_("Error"), wxOK | wxICON_ERROR,MainFrame::Get());
+            wxMessageBox(_("Sorry, the fast preview window requires a system which supports OpenGL version 1.1 with the GL_ARB_multitexture extension.\nThe fast preview cannot be opened.\n\nHugin has been configured to start without fast preview.\nPlease restart Hugin."),_("Error"), wxOK | wxICON_ERROR);
             return;
         }
 
         setUp();
+#ifdef __WXGTK__
+        if(this==frame->getPreview())
+        {
+            frame->getOverview()->setUp();
+        }
+        else
+        {
+            frame->getPreview()->setUp();
+        };
+#endif
     }
 }
 
 void GLPreview::setUp()
 {
     DEBUG_DEBUG("Preview Setup");
+    if (m_toolsInitialized)
+    {
+        return;
+    };
+    m_toolsInitialized = true;
     // we need something to store the state of the view and control updates
-    if (!m_view_state) {
+    if (!m_view_state)
+    {
         GLint countMultiTexture;
         glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB,&countMultiTexture);
         m_view_state = new ViewState(m_pano, countMultiTexture>1);
     }
+    ++m_view_state_observer;
     m_visualization_state = new VisualizationState(m_pano, m_view_state, this, RefreshWrapper, this, (PreviewMeshManager*) NULL);
     //Start the tools going:
     PreviewToolHelper *helper = new PreviewToolHelper(m_pano, m_visualization_state, frame);
-    m_tool_helper = (ToolHelper*) helper;
+    m_tool_helper = static_cast<ToolHelper*>(helper);
     frame->MakePreviewTools(helper);
     // now make a renderer
     m_renderer =  new GLPreviewRenderer(m_pano, m_view_state->GetTextureManager(),
                                  m_visualization_state->GetMeshManager(),
                                  m_visualization_state, helper, m_background_color);
-    // check, if gpu supports multitextures
-    // fill blend mode choice box in fast preview window
-    // we can fill it just now, because we need a OpenGL context, which was created now,
-    // to check if all necessary extentions are available
-    frame->FillBlendChoice();
+}
+
+GLOverview::~GLOverview()
+{
+    if (m_renderer)
+    {
+        if (m_renderer == plane_m_renderer)
+        {
+            delete panosphere_m_tool_helper;
+            delete panosphere_m_renderer;
+            delete panosphere_m_visualization_state;
+        }
+        else
+        {
+            delete plane_m_tool_helper;
+            delete plane_m_renderer;
+            delete plane_m_visualization_state;
+        };
+    };
 }
 
 void GLOverview::setUp()
 {
-DEBUG_DEBUG("Overview Setup");
+    DEBUG_DEBUG("Overview Setup");
+    if (m_toolsInitialized)
+    {
+        return;
+    };
+    m_toolsInitialized = true;
     // we need something to store the state of the view and control updates
-    if (!m_view_state) {
+    if (!m_view_state)
+    {
         GLint countMultiTexture;
         glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB,&countMultiTexture);
         m_view_state = new ViewState(m_pano, countMultiTexture>1);
     }
+    ++m_view_state_observer;
 
     panosphere_m_visualization_state = new PanosphereOverviewVisualizationState(m_pano, m_view_state, this, RefreshWrapper, this);
     plane_m_visualization_state = new PlaneOverviewVisualizationState(m_pano, m_view_state, this, RefreshWrapper, this);
@@ -301,7 +324,7 @@ void GLViewer::RedrawE(wxPaintEvent& e)
 void GLViewer::RefreshWrapper(void * obj)
 {
     DEBUG_DEBUG("REFRESH WRAPPER");
-    GLViewer* self = (GLViewer*) obj;
+    GLViewer* self = static_cast<GLViewer*>(obj);
     self->Refresh();
 }
 
@@ -316,7 +339,6 @@ void GLViewer::Resized(wxSizeEvent& e)
    
     if (frame->CanResize()) {
         DEBUG_DEBUG("RESIZED_IN");
-        wxGLCanvas::OnSize(e);
         if(!IsShown()) return;
         // if we have a render at this point, tell it the new size.
         DEBUG_DEBUG("RESIZED_IN_SHOWN");
@@ -336,11 +358,17 @@ void GLViewer::Redraw()
 {
     // get the renderer to redraw the OpenGL stuff
     if(!m_renderer) return;
+    // don't redraw if we are in middle of a pending change of the panorama object
+    if(m_pano->hasPendingChanges()) return;
     DEBUG_INFO("Rendering.");
     
     // we should use the window background colour outside the panorama
     // FIXME shouldn't this work on textured backrounds?
-    wxColour col = wxSystemSettings::GetColour(wxSYS_COLOUR_APPWORKSPACE);
+#if defined __WXMAC__ && wxCHECK_VERSION(3,1,0) 
+    wxColour col(128,128,128);
+#else
+    wxColour col = wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE);
+#endif
     m_renderer->SetBackground(col.Red(), col.Green(), col.Blue());
     if (m_visualization_state->RequireRecalculateViewport())
     {
@@ -370,6 +398,13 @@ void GLViewer::MouseMotion(wxMouseEvent& e)
                               (int) e.m_y - offset.y, e);
 }
 
+void GLViewer::MouseEnter(wxMouseEvent & e)
+{
+    if(m_renderer)
+        m_tool_helper->MouseEnter((int) e.m_x - offset.x,
+                              (int) e.m_y - offset.y, e);
+}
+
 void GLViewer::MouseLeave(wxMouseEvent & e)
 {
     if(m_renderer)
@@ -411,6 +446,11 @@ void GLViewer::KeyUp(wxKeyEvent& e)
         m_tool_helper->KeypressEvent(e.GetKeyCode(), e.GetModifiers(), false);
 }
 
+void GLViewer::MarkToolsDirty()
+{
+    m_tool_helper->MarkDirty();
+}
+
 void GLViewer::SetViewerBackground(wxColour col)
 {
     this->m_background_color = col;
@@ -438,3 +478,4 @@ void GLOverview::SetMode(OverviewMode mode)
     }
 }
 
+
