@@ -53,6 +53,8 @@ enum
     ID_UNSELECT_ALL,
     ID_SELECT_LENS_STACK,
     ID_UNSELECT_LENS_STACK,
+    ID_ACTIVATE_IMAGE,
+    ID_DEACTIVATE_IMAGE,
     ID_OPERATION_START=wxID_HIGHEST+300
 };
 
@@ -67,6 +69,8 @@ BEGIN_EVENT_TABLE(ImagesTreeCtrl, wxTreeListCtrl)
     EVT_MENU(ID_UNSELECT_ALL, ImagesTreeCtrl::OnUnselectAll)
     EVT_MENU(ID_SELECT_LENS_STACK, ImagesTreeCtrl::OnSelectLensStack)
     EVT_MENU(ID_UNSELECT_LENS_STACK, ImagesTreeCtrl::OnUnselectLensStack)
+    EVT_MENU(ID_ACTIVATE_IMAGE, ImagesTreeCtrl::OnActivateImage)
+    EVT_MENU(ID_DEACTIVATE_IMAGE, ImagesTreeCtrl::OnDeactivateImage)
     EVT_MENU_RANGE(ID_OPERATION_START, ID_OPERATION_START+50, ImagesTreeCtrl::OnExecuteOperation)
     EVT_TREE_BEGIN_DRAG(-1, ImagesTreeCtrl::OnBeginDrag)
     EVT_LEFT_UP(ImagesTreeCtrl::OnLeftUp)
@@ -99,6 +103,7 @@ ImagesTreeCtrl::ImagesTreeCtrl()
     m_displayMode=DISPLAY_GENERAL;
     m_optimizerMode=false;
     m_needsUpdate=true;
+    m_markDisabledImages = false;
 }
 
 bool ImagesTreeCtrl::Create(wxWindow* parent, wxWindowID id,
@@ -342,6 +347,10 @@ void ImagesTreeCtrl::panoramaImagesChanged(HuginBase::Panorama &pano, const Hugi
     {
         UpdateOptimizerVariables();
     };
+    if (m_markDisabledImages)
+    {
+        UpdateItemFont();
+    };
 
     Thaw();
     m_needsUpdate = false;
@@ -399,86 +408,11 @@ void ImagesTreeCtrl::UpdateImageText(wxTreeItemId item)
     SetItemText(item, m_columnMap["maker"], wxString(img.getExifMake().c_str(), wxConvLocal));
     SetItemText(item, m_columnMap["model"], wxString(img.getExifModel().c_str(), wxConvLocal));
     SetItemText(item, m_columnMap["lens"], wxString(img.getExifLens().c_str(), wxConvLocal));
-    struct tm exifdatetime;
-    if(img.getExifDateTime(&exifdatetime)==0)
-    {
-        wxDateTime s_datetime=wxDateTime(exifdatetime);
-        s=s_datetime.Format();
-    }
-    else
-    {
-        s = wxString(img.getExifDate().c_str(),wxConvLocal);
-    }
-    SetItemText(item, m_columnMap["date"], s);
-
-    if(img.getExifFocalLength()>0.0)
-    {
-        if(img.getExifFocalLength35()>0.0)
-        {
-            s = wxString::Format(wxT("%0.1f mm (%0.0f mm)"),img.getExifFocalLength(),img.getExifFocalLength35());
-        }
-        else
-        {
-            s = wxString::Format(wxT("%0.1f mm"),img.getExifFocalLength());
-        };
-    }
-    else
-    {
-        s = wxEmptyString;
-    };
-    SetItemText(item, m_columnMap["focallength"], s);
-
-    if(img.getExifAperture()>0)
-    {
-        s=wxString::Format(wxT("F%.1f"),img.getExifAperture());
-    }
-    else
-    {
-        s=wxEmptyString;
-    };
-    SetItemText(item, m_columnMap["aperture"], s);
-
-    if(img.getExifExposureTime()>0.5)
-    {
-        if(img.getExifExposureTime()>=1.0) 
-        {
-            if(img.getExifExposureTime()>=10.0) 
-            {
-                s=wxString::Format(wxT("%3.0f s"),img.getExifExposureTime());
-            }
-            else
-            {
-                s=wxString::Format(wxT("%1.1f s"),img.getExifExposureTime());
-            }
-        }
-        else
-        {
-            s=wxString::Format(wxT("%1.2f s"),img.getExifExposureTime());
-        }
-    }
-    else
-    {
-        if (img.getExifExposureTime() > 1e-9)
-        {
-            s=wxString::Format(wxT("1/%0.0f s"),1.0/img.getExifExposureTime());
-        } 
-        else
-        {
-            //Sanity
-            s=wxT("");
-        }
-    }
-    SetItemText(item, m_columnMap["time"], s);
-
-    if(img.getExifISO()>0)
-    {
-        s=wxString::Format(wxT("%0.0f"),img.getExifISO());
-    }
-    else
-    {
-        s=wxEmptyString;
-    };
-    SetItemText(item, m_columnMap["iso"], s);
+    SetItemText(item, m_columnMap["date"], FormatString::GetExifDateTime(&img));
+    SetItemText(item, m_columnMap["focallength"], FormatString::GetFocalLength(&img));
+    SetItemText(item, m_columnMap["aperture"], FormatString::GetAperture(&img));
+    SetItemText(item, m_columnMap["time"], FormatString::GetExposureTime(&img));
+    SetItemText(item, m_columnMap["iso"], FormatString::GetIso(&img));
 
     if (m_groupMode == GROUP_STACK && (img.YawisLinked() || isSingleImage))
     {
@@ -497,9 +431,9 @@ void ImagesTreeCtrl::UpdateImageText(wxTreeItemId item)
         SetItemText(item, m_columnMap["y"], hugin_utils::doubleTowxString(img.getYaw(), m_degDigits));
         SetItemText(item, m_columnMap["p"], hugin_utils::doubleTowxString(img.getPitch(), m_degDigits));
         SetItemText(item, m_columnMap["r"], hugin_utils::doubleTowxString(img.getRoll(), m_degDigits));
-        SetItemText(item, m_columnMap["TrX"], hugin_utils::doubleTowxString(img.getX(), m_degDigits));
-        SetItemText(item, m_columnMap["TrY"], hugin_utils::doubleTowxString(img.getY(), m_degDigits));
-        SetItemText(item, m_columnMap["TrZ"], hugin_utils::doubleTowxString(img.getZ(), m_degDigits));
+        SetItemText(item, m_columnMap["TrX"], hugin_utils::doubleTowxString(img.getX(), m_distDigits));
+        SetItemText(item, m_columnMap["TrY"], hugin_utils::doubleTowxString(img.getY(), m_distDigits));
+        SetItemText(item, m_columnMap["TrZ"], hugin_utils::doubleTowxString(img.getZ(), m_distDigits));
         SetItemText(item, m_columnMap["Tpy"], hugin_utils::doubleTowxString(img.getTranslationPlaneYaw(), m_degDigits));
         SetItemText(item, m_columnMap["Tpp"], hugin_utils::doubleTowxString(img.getTranslationPlanePitch(), m_degDigits));
         wxString text=_("not active");
@@ -918,7 +852,7 @@ void ImagesTreeCtrl::UpdateOptimizerVariables()
         {
             imgNrs.insert(data->GetImgNr());
         };
-        if(imgNrs.size()>0)
+        if(!imgNrs.empty())
         {
             for(size_t i=0;i<GetColumnCount();i++)
             {
@@ -984,6 +918,12 @@ HuginBase::UIntSet ImagesTreeCtrl::GetSelectedImages()
         };
     }
     return imgs;
+}
+
+void ImagesTreeCtrl::MarkActiveImages(const bool markActive)
+{
+    m_markDisabledImages = markActive;
+    UpdateItemFont();
 };
 
 void ImagesTreeCtrl::OnColumnWidthChange( wxListEvent & e )
@@ -1069,6 +1009,8 @@ void ImagesTreeCtrl::SetGroupMode(GroupMode newGroup)
             fill_set(imgs,0,m_pano->getNrOfImages()-1);
         };
         panoramaImagesChanged(*m_pano,imgs);
+        // reset already modified tag
+        m_needsUpdate = true;
         ExpandAll(m_root);
     };
 };
@@ -1143,8 +1085,20 @@ void ImagesTreeCtrl::OnContextMenu(wxTreeEvent & e)
         {
             bool emptyText=GetItemText(e.GetItem(),m_selectedColumn).IsEmpty();
             ImagesTreeData* data=static_cast<ImagesTreeData*>(GetItemData(e.GetItem()));
-            if((m_groupMode==GROUP_LENS && m_variableVector[m_selectedColumn]!=HuginBase::ImageVariableGroup::IVE_Yaw) ||
-               (m_groupMode==GROUP_STACK && m_variableVector[m_selectedColumn]==HuginBase::ImageVariableGroup::IVE_Yaw) )
+            bool varIsLinkable = false;
+            if (m_groupMode == GROUP_LENS)
+            {
+                varIsLinkable = m_variableVector[m_selectedColumn] != HuginBase::ImageVariableGroup::IVE_Yaw &&
+                    set_contains(HuginBase::ConstStandardImageVariableGroups::getLensVariables(), m_variableVector[m_selectedColumn]);
+            }
+            else
+            {
+                if (m_groupMode == GROUP_STACK)
+                {
+                    varIsLinkable = m_variableVector[m_selectedColumn] == HuginBase::ImageVariableGroup::IVE_Yaw;
+                }
+            };
+            if(varIsLinkable)
             {
                 if(data->IsGroup())
                 {
@@ -1194,6 +1148,26 @@ void ImagesTreeCtrl::OnContextMenu(wxTreeEvent & e)
             }
         };
         menu.Append(ID_EDIT, _("Edit image variables..."));
+        if (m_markDisabledImages)
+        {
+            const HuginBase::UIntSet selectedImages = GetSelectedImages();
+            if (selectedImages.size() == 1)
+            {
+                if (m_pano->getImage(*selectedImages.begin()).getActive())
+                {
+                    menu.Append(ID_DEACTIVATE_IMAGE, _("Deactivate image"));
+                }
+                else
+                {
+                    menu.Append(ID_ACTIVATE_IMAGE, _("Activate image"));
+                };
+            }
+            else
+            {
+                menu.Append(ID_ACTIVATE_IMAGE, _("Activate images"));
+                menu.Append(ID_DEACTIVATE_IMAGE, _("Deactivate images"));
+            };
+        };
     }
     else
     {
@@ -1279,6 +1253,89 @@ void ImagesTreeCtrl::GenerateSubMenu(wxMenu* menu, PanoOperation::PanoOperationV
             id++;
         }
     };
+}
+
+void ImagesTreeCtrl::UpdateItemFont()
+{
+    const wxColour normalColour = wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT);
+    const wxColour disabledColour = wxSystemSettings::GetColour(wxSYS_COLOUR_GRAYTEXT);
+    if (m_markDisabledImages)
+    {
+        wxTreeItemIdValue cookie;
+        wxTreeItemId item = GetFirstChild(m_root, cookie);
+        while (item.IsOk())
+        {
+            ImagesTreeData* itemData = static_cast<ImagesTreeData*>(GetItemData(item));
+            bool isActive = true;
+            if (itemData->IsGroup())
+            {
+                wxTreeItemIdValue childCookie;
+                wxTreeItemId child = GetFirstChild(item, childCookie);
+                while (child.IsOk())
+                {
+                    ImagesTreeData* childItemData = static_cast<ImagesTreeData*>(GetItemData(child));
+                    isActive = m_pano->getImage(childItemData->GetImgNr()).getActive();
+                    if (isActive)
+                    {
+                        break;
+                    };
+                    child = GetNextChild(item, childCookie);
+                };
+            }
+            else
+            {
+                isActive = m_pano->getImage(itemData->GetImgNr()).getActive();
+            };
+            if (isActive)
+            {
+                SetItemTextColour(item, normalColour);
+            }
+            else
+            {
+                SetItemTextColour(item, disabledColour);
+            };
+            if(itemData->IsGroup())
+            {
+                wxTreeItemIdValue childCookie;
+                wxTreeItemId child = GetFirstChild(item, childCookie);
+                while (child.IsOk())
+                {
+                    ImagesTreeData* childItemData = static_cast<ImagesTreeData*>(GetItemData(child));
+                    if (m_pano->getImage(childItemData->GetImgNr()).getActive())
+                    {
+                        SetItemTextColour(child, normalColour);
+                    }
+                    else
+                    {
+                        SetItemTextColour(child, disabledColour);
+                    };
+                    child = GetNextChild(item, childCookie);
+                };
+            };
+            item = GetNextChild(m_root, cookie);
+        };
+    }
+    else
+    {
+        wxTreeItemIdValue cookie;
+        wxTreeItemId item = GetFirstChild(m_root, cookie);
+        while (item.IsOk())
+        {
+            ImagesTreeData* itemData=static_cast<ImagesTreeData*>(GetItemData(item));
+            SetItemTextColour(item, normalColour);
+            if (itemData->IsGroup())
+            {
+                wxTreeItemIdValue childCookie;
+                wxTreeItemId child = GetFirstChild(item, childCookie);
+                while (child.IsOk())
+                {
+                    SetItemTextColour(child, normalColour);
+                    child = GetNextChild(item, childCookie);
+                };
+            };
+            item = GetNextChild(m_root, cookie);
+        };
+    };
 };
 
 void ImagesTreeCtrl::OnHeaderContextMenu(wxListEvent & e)
@@ -1292,12 +1349,32 @@ void ImagesTreeCtrl::OnHeaderContextMenu(wxListEvent & e)
         PopupMenu(&menu);
     };
     e.Skip();
+}
+void ImagesTreeCtrl::OnActivateImage(wxCommandEvent & e)
+{
+    HuginBase::UIntSet activeImages = m_pano->getActiveImages();
+    const HuginBase::UIntSet selectedImages = GetSelectedImages();
+    std::copy(selectedImages.begin(), selectedImages.end(), std::inserter(activeImages, activeImages.end()));
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetActiveImagesCmd(*m_pano, activeImages)
+    );
+};
+
+void ImagesTreeCtrl::OnDeactivateImage(wxCommandEvent & e)
+{
+    HuginBase::UIntSet activeImages = m_pano->getActiveImages();
+    const HuginBase::UIntSet selectedImages = GetSelectedImages();
+    HuginBase::UIntSet newActiveImages;
+    std::set_difference(activeImages.begin(), activeImages.end(), selectedImages.begin(), selectedImages.end(), std::inserter(newActiveImages, newActiveImages.end()));
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetActiveImagesCmd(*m_pano, newActiveImages)
+    );
 };
 
 void ImagesTreeCtrl::UnLinkImageVariables(bool linked)
 {
     HuginBase::UIntSet images=GetSelectedImages();
-    if(images.size()>0 && m_variableVector[m_selectedColumn]!=HuginBase::ImageVariableGroup::IVE_Filename)
+    if(!images.empty() && m_variableVector[m_selectedColumn]!=HuginBase::ImageVariableGroup::IVE_Filename)
     {
         std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> variables;
         variables.insert(m_variableVector[m_selectedColumn]);
@@ -1339,7 +1416,7 @@ void ImagesTreeCtrl::OnUnlinkImageVariables(wxCommandEvent &e)
 void ImagesTreeCtrl::OnEditImageVariables(wxCommandEvent &e)
 {
     HuginBase::UIntSet imgs=GetSelectedImages();
-    if(imgs.size()>0)
+    if(!imgs.empty())
     {
         ImageVariableDialog dlg(this, m_pano, imgs);
         dlg.SetGuiLevel(m_guiLevel);
@@ -1378,7 +1455,7 @@ void ImagesTreeCtrl::OnBeginDrag(wxTreeEvent &e)
     if(m_pano->getNrOfImages()>0 && !m_dragging)
     {
         m_draggingImages=GetSelectedImages();
-        if(m_draggingImages.size()>0)
+        if(!m_draggingImages.empty())
         {
             if((m_groupMode==GROUP_NONE && m_draggingImages.size()==1 && !ctrlPressed) ||
                 m_groupMode==GROUP_LENS || m_groupMode==GROUP_STACK)
@@ -1532,21 +1609,32 @@ void ImagesTreeCtrl::OnLeftUp(wxMouseEvent &e)
             if (item.IsOk() && item == m_leftDownItem && col == m_leftDownColumn && (flags & wxTREE_HITTEST_ONITEMICON))
             {
                 HuginBase::UIntSet imgs;
-                ImagesTreeData* data = static_cast<ImagesTreeData*>(GetItemData(m_leftDownItem));
-                if (data->IsGroup())
+                const bool markSelected = IsSelected(m_leftDownItem);
+                if (markSelected)
                 {
-                    wxTreeItemIdValue cookie;
-                    wxTreeItemId childItem = GetFirstChild(m_leftDownItem, cookie);
-                    while (childItem.IsOk())
-                    {
-                        data = static_cast<ImagesTreeData*>(GetItemData(childItem));
-                        imgs.insert(data->GetImgNr());
-                        childItem = GetNextChild(m_leftDownItem, cookie);
-                    };
+                    // if the user clicked on one of the selected pass the selection to all selected images
+                    imgs = GetSelectedImages();
                 }
                 else
                 {
-                    imgs.insert(data->GetImgNr());
+                    // the user clicked on a non-selected images, work only on this image
+                    // or all images of the clicked lens/stack
+                    ImagesTreeData* data = static_cast<ImagesTreeData*>(GetItemData(m_leftDownItem));
+                    if (data->IsGroup())
+                    {
+                        wxTreeItemIdValue cookie;
+                        wxTreeItemId childItem = GetFirstChild(m_leftDownItem, cookie);
+                        while (childItem.IsOk())
+                        {
+                            data = static_cast<ImagesTreeData*>(GetItemData(childItem));
+                            imgs.insert(data->GetImgNr());
+                            childItem = GetNextChild(m_leftDownItem, cookie);
+                        };
+                    }
+                    else
+                    {
+                        imgs.insert(data->GetImgNr());
+                    };
                 };
                 HuginBase::OptimizeVector optVec = m_pano->getOptimizeVector();
                 std::set<std::string> var;
@@ -1588,14 +1676,22 @@ void ImagesTreeCtrl::OnLeftUp(wxMouseEvent &e)
                     };
                 };
                 bool deactivate = false;
-                for (std::set<std::string>::const_iterator varIt = var.begin(); varIt != var.end(); ++varIt)
+                if (markSelected)
                 {
-                    //search, if image variable is marked for optimise for at least one image of group
-                    for (HuginBase::UIntSet::const_iterator imgIt = imgs.begin(); imgIt != imgs.end() && !deactivate; ++imgIt)
+                    // check which state the clicked image have
+                    deactivate = GetItemImage(m_leftDownItem, m_leftDownColumn) & 1;
+                }
+                else
+                {
+                    for (std::set<std::string>::const_iterator varIt = var.begin(); varIt != var.end(); ++varIt)
                     {
-                        if (set_contains(optVec[*imgIt], *varIt))
+                        //search, if image variable is marked for optimise for at least one image of group
+                        for (HuginBase::UIntSet::const_iterator imgIt = imgs.begin(); imgIt != imgs.end() && !deactivate; ++imgIt)
                         {
-                            deactivate = true;
+                            if (set_contains(optVec[*imgIt], *varIt))
+                            {
+                                deactivate = true;
+                            };
                         };
                     };
                 };
@@ -1777,12 +1873,22 @@ void ImagesTreeCtrl::SelectAllParameters(bool select, bool allImages)
             if(select)
             {
                 if((*it=="y" || *it=="p" || *it=="r" || *it=="TrX" || *it=="TrY" || *it=="TrZ" || *it=="Tpy" || *it=="Tpp") && 
-                    *img==m_pano->getOptions().optimizeReferenceImage)
+                    (*img==m_pano->getOptions().optimizeReferenceImage || m_pano->getImage(*img).YawisLinkedWith(m_pano->getImage(m_pano->getOptions().optimizeReferenceImage))))
                 {
                     optVec[*img].erase(*it);
                     continue;
                 };
-                if((*it=="Eev" || *it=="Er" || *it=="Eb") && *img==m_pano->getOptions().colorReferenceImage)
+                if(*it=="Eev" && (*img==m_pano->getOptions().colorReferenceImage || m_pano->getImage(*img).ExposureValueisLinkedWith(m_pano->getImage(m_pano->getOptions().colorReferenceImage))))
+                {
+                    optVec[*img].erase(*it);
+                    continue;
+                };
+                if (*it == "Er" && (*img == m_pano->getOptions().colorReferenceImage || m_pano->getImage(*img).WhiteBalanceRedisLinkedWith(m_pano->getImage(m_pano->getOptions().colorReferenceImage))))
+                {
+                    optVec[*img].erase(*it);
+                    continue;
+                };
+                if (*it == "Eb" && (*img == m_pano->getOptions().colorReferenceImage || m_pano->getImage(*img).WhiteBalanceBlueisLinkedWith(m_pano->getImage(m_pano->getOptions().colorReferenceImage))))
                 {
                     optVec[*img].erase(*it);
                     continue;
@@ -1833,7 +1939,7 @@ void ImagesTreeCtrl::OnChar(wxTreeEvent &e)
         case WXK_DELETE:
             {
                 HuginBase::UIntSet imgs=GetSelectedImages();
-                if(imgs.size()>0)
+                if(!imgs.empty())
                 {
                     PanoCommand::GlobalCmdHist::getInstance().addCommand(
                         new PanoCommand::RemoveImagesCmd(*m_pano, imgs)
