@@ -32,12 +32,18 @@
 #include "algorithms/optimizer/ImageGraph.h"
 #include "algorithms/control_points/CleanCP.h"
 #include "celeste/Celeste.h"
+#ifdef _WIN32
+// workaround for a conflict between exiv2 and wxWidgets/CMake built
+#define HAVE_PID_T 1
+#endif
 #include <exiv2/exif.hpp>
 #include <exiv2/image.hpp>
 #include "base_wx/LensTools.h"
 #include "base_wx/wxLensDB.h"
 #include "hugin/ResetDialog.h"
+#include "hugin/ChangeImageVariableDialog.h"
 #include "hugin/MainFrame.h"
+#include "hugin/RawImport.h"
 #include <vigra_ext/openmp_vigra.h>
 #include <vigra_ext/cms.h>
 
@@ -75,7 +81,7 @@ bool PanoSingleImageOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::U
 
 bool PanoMultiImageOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
-    return images.size()>0;
+    return !images.empty();
 };
 
 /** small function to show add image dialog
@@ -83,14 +89,14 @@ bool PanoMultiImageOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UI
   * @param files vector, to which the selected valid filenames will be added
   * @returns true, if a valid image was selected, otherwise false
   */
-bool AddImageDialog(wxWindow* parent, std::vector<std::string>& files)
+bool AddImageDialog(wxWindow* parent, std::vector<std::string>& files, bool& withRaws)
 {
     // get stored path
     wxConfigBase* config = wxConfigBase::Get();
     wxString path = config->Read(wxT("/actualPath"), wxT(""));
     wxFileDialog dlg(parent,_("Add images"),
                      path, wxT(""),
-                     HUGIN_WX_FILE_IMG_FILTER,
+                     withRaws ? GetFileDialogImageAndRawFilters() : GetFileDialogImageFilters(),
                      wxFD_OPEN | wxFD_MULTIPLE | wxFD_FILE_MUST_EXIST | wxFD_PREVIEW, wxDefaultPosition);
     dlg.SetDirectory(path);
 
@@ -101,19 +107,42 @@ bool AddImageDialog(wxWindow* parent, std::vector<std::string>& files)
       img_ext = config->Read(wxT("lastImageType")).c_str();
     }
     if (img_ext == wxT("all images"))
-      dlg.SetFilterIndex(0);
-    else if (img_ext == wxT("jpg"))
-      dlg.SetFilterIndex(1);
-    else if (img_ext == wxT("tiff"))
-      dlg.SetFilterIndex(2);
-    else if (img_ext == wxT("png"))
-      dlg.SetFilterIndex(3);
-    else if (img_ext == wxT("hdr"))
-      dlg.SetFilterIndex(4);
-    else if (img_ext == wxT("exr"))
-      dlg.SetFilterIndex(5);
-    else if (img_ext == wxT("all files"))
-      dlg.SetFilterIndex(6);
+    {
+        dlg.SetFilterIndex(0);
+    }
+    else
+    {
+        if (!withRaws)
+        {
+            if (img_ext == wxT("jpg"))
+                dlg.SetFilterIndex(1);
+            else if (img_ext == wxT("tiff"))
+                dlg.SetFilterIndex(2);
+            else if (img_ext == wxT("png"))
+                dlg.SetFilterIndex(3);
+            else if (img_ext == wxT("hdr"))
+                dlg.SetFilterIndex(4);
+            else if (img_ext == wxT("exr"))
+                dlg.SetFilterIndex(5);
+            else if (img_ext == wxT("all files"))
+                dlg.SetFilterIndex(6);
+        }
+        else
+        {
+            if (img_ext == wxT("all raws"))
+                dlg.SetFilterIndex(1);
+            else if (img_ext == wxT("jpg"))
+                dlg.SetFilterIndex(2);
+            else if (img_ext == wxT("tiff"))
+                dlg.SetFilterIndex(3);
+            else if (img_ext == wxT("png"))
+                dlg.SetFilterIndex(4);
+            else if (img_ext == wxT("hdr"))
+                dlg.SetFilterIndex(5);
+            else if (img_ext == wxT("exr"))
+                dlg.SetFilterIndex(6);
+        };
+    };
 
     // call the file dialog
     if (dlg.ShowModal() == wxID_OK)
@@ -130,16 +159,33 @@ bool AddImageDialog(wxWindow* parent, std::vector<std::string>& files)
         config->Write(wxT("/actualPath"), dlg.GetDirectory());
 #endif
         // save the image extension
-        switch (dlg.GetFilterIndex())
+        if (!withRaws)
         {
-            case 0: config->Write(wxT("lastImageType"), wxT("all images")); break;
-            case 1: config->Write(wxT("lastImageType"), wxT("jpg")); break;
-            case 2: config->Write(wxT("lastImageType"), wxT("tiff")); break;
-            case 3: config->Write(wxT("lastImageType"), wxT("png")); break;
-            case 4: config->Write(wxT("lastImageType"), wxT("hdr")); break;
-            case 5: config->Write(wxT("lastImageType"), wxT("exr")); break;
-            case 6: config->Write(wxT("lastImageType"), wxT("all files")); break;
+            switch (dlg.GetFilterIndex())
+            {
+                case 0: config->Write(wxT("lastImageType"), wxT("all images")); break;
+                case 1: config->Write(wxT("lastImageType"), wxT("jpg")); break;
+                case 2: config->Write(wxT("lastImageType"), wxT("tiff")); break;
+                case 3: config->Write(wxT("lastImageType"), wxT("png")); break;
+                case 4: config->Write(wxT("lastImageType"), wxT("hdr")); break;
+                case 5: config->Write(wxT("lastImageType"), wxT("exr")); break;
+                case 6: config->Write(wxT("lastImageType"), wxT("all files")); break;
+            };
         }
+        else
+        {
+            switch (dlg.GetFilterIndex())
+            {
+                case 0: config->Write(wxT("lastImageType"), wxT("all images")); break;
+                case 1: config->Write(wxT("lastImageType"), wxT("all raws")); break;
+                case 2: config->Write(wxT("lastImageType"), wxT("jpg")); break;
+                case 3: config->Write(wxT("lastImageType"), wxT("tiff")); break;
+                case 4: config->Write(wxT("lastImageType"), wxT("png")); break;
+                case 5: config->Write(wxT("lastImageType"), wxT("hdr")); break;
+                case 6: config->Write(wxT("lastImageType"), wxT("exr")); break;
+            }
+            withRaws = dlg.GetFilterIndex() == 1;
+        };
 
         //check for forbidden/non working chars
         wxArrayString invalidFiles;
@@ -150,7 +196,7 @@ bool AddImageDialog(wxWindow* parent, std::vector<std::string>& files)
                invalidFiles.Add(Pathnames[i]);
            };
         };
-        if(invalidFiles.size()>0)
+        if(!invalidFiles.empty())
         {
             ShowFilenameWarning(parent, invalidFiles);
             return false;
@@ -172,11 +218,46 @@ wxString AddImageOperation::GetLabel()
 PanoCommand::PanoCommand* AddImageOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
     std::vector<std::string> files;
-    if(AddImageDialog(parent, files))
+    bool withRaws = true;
+    if(AddImageDialog(parent, files, withRaws))
     {
-        if(files.size()>0)
+        if(!files.empty())
         {
-            return new PanoCommand::wxAddImagesCmd(pano,files);
+            if (withRaws)
+            {
+                if (files.size() == 1)
+                {
+                    wxMessageDialog message(parent, _("You selected only one raw file. This is not recommended.\nAll raw files should be converted at once."),
+#ifdef _WIN32
+                        _("Hugin"),
+#else
+                        wxT(""),
+#endif
+                        wxICON_EXCLAMATION | wxYES_NO );
+                    message.SetYesNoLabels(_("Convert anyway."), _("Let me select several raw files."));
+                    if (message.ShowModal() == wxID_NO)
+                    {
+                        // post new add image event to open dialog again
+                        wxCommandEvent newAddEvent(wxEVT_COMMAND_BUTTON_CLICKED, XRCID("action_add_images"));
+                        MainFrame::Get()->GetEventHandler()->AddPendingEvent(newAddEvent);
+                        return NULL;
+                    };
+                };
+                RawImportDialog dlg(parent, &pano, files);
+                // check that raw files are from same camera and that all can be read
+                if (dlg.CheckRawFiles())
+                {
+                    // now show dialog
+                    if (dlg.ShowModal() == wxID_OK)
+                    {
+                        return dlg.GetPanoCommand();
+                    };
+                };
+            }
+            else
+            {
+                return new PanoCommand::wxAddImagesCmd(pano, files);
+            };
         };
     };
     return NULL;
@@ -264,12 +345,13 @@ PanoCommand::PanoCommand* AddImagesSeriesOperation::GetInternalCommand(wxWindow*
     std::vector<std::string> files;
     if(pano.getNrOfImages()==0)
     {
-        if(!AddImageDialog(parent,files))
+        bool withRaws = false;
+        if(!AddImageDialog(parent,files, withRaws))
         {
             return NULL;
         };
         //just in case
-        if(files.size()==0)
+        if(files.empty())
         {
             return NULL;
         };
@@ -357,7 +439,7 @@ PanoCommand::PanoCommand* AddImagesSeriesOperation::GetInternalCommand(wxWindow*
         }
     }
 
-    if(files.size()>0)
+    if(!files.empty())
     {
         // sort files by date
         sortbytime spred(timeMap);
@@ -379,6 +461,30 @@ PanoCommand::PanoCommand* AddImagesSeriesOperation::GetInternalCommand(wxWindow*
     };
 };
 
+wxString ImageVariablesExpressionOperation::GetLabel()
+{
+    return _("Manipulate image variables...");
+}
+
+PanoCommand::PanoCommand * ImageVariablesExpressionOperation::GetInternalCommand(wxWindow * parent, HuginBase::Panorama & pano, HuginBase::UIntSet images)
+{
+    ImageVariablesExpressionDialog dlg(parent, &pano);
+    if (dlg.ShowModal() == wxID_OK)
+    {
+        const std::string expression = dlg.GetExpression();
+        if (!expression.empty())
+        {
+            return new PanoCommand::UpdateVariablesByParseExpression(pano, expression);
+        };
+    };
+    return NULL;
+}
+
+bool ImageVariablesExpressionOperation::IsEnabled(HuginBase::Panorama & pano, HuginBase::UIntSet images, GuiLevel guiLevel)
+{
+    return pano.getNrOfImages() > 0 && guiLevel >= GuiLevel::GUI_ADVANCED;
+}
+
 wxString RemoveImageOperation::GetLabel()
 {
     return _("Remove selected image(s)");
@@ -425,7 +531,7 @@ PanoCommand::PanoCommand* ChangeColorAnchorImageOperation::GetInternalCommand(wx
 
 bool NewLensOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
-    if(pano.getNrOfImages()==0 || images.size()==0)
+    if(pano.getNrOfImages()==0 || images.empty())
     {
         return false;
     }
@@ -448,13 +554,13 @@ PanoCommand::PanoCommand* NewLensOperation::GetInternalCommand(wxWindow* parent,
 
 bool ChangeLensOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
-    if(pano.getNrOfImages()==0 || images.size()==0)
+    if(pano.getNrOfImages()==0 || images.empty())
     {
         return false;
     }
     else
     {
-        //project must have more than 1 lens before you can assign an other lens number
+        //project must have more than 1 lens before you can assign another lens number
         HuginBase::StandardImageVariableGroups variableGroups(pano);
         return variableGroups.getLenses().getNumberOfParts() > 1;
     };
@@ -593,7 +699,7 @@ bool RemoveControlPointsOperation::IsEnabled(HuginBase::Panorama& pano, HuginBas
 PanoCommand::PanoCommand* RemoveControlPointsOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
     HuginBase::UIntSet selImages;
-    if(images.size()==0)
+    if(images.empty())
     {
         fill_set(selImages,0,pano.getNrOfCtrlPoints()-1);
     }
@@ -610,7 +716,7 @@ PanoCommand::PanoCommand* RemoveControlPointsOperation::GetInternalCommand(wxWin
             cpsToDelete.insert(it - cps.begin());
         }
     }
-    if(cpsToDelete.size()==0)
+    if(cpsToDelete.empty())
     {
         wxMessageBox(_("Selected images have no control points."),
 #ifdef __WXMSW__
@@ -680,7 +786,7 @@ PanoCommand::PanoCommand* CleanControlPointsOperation::GetInternalCommand(wxWind
     {
         //now run the second step
         HuginBase::UIntSet removedCP2=getCPoutsideLimit(newPano, 2.0);
-        if(removedCP2.size()>0)
+        if(!removedCP2.empty())
         {
             for(HuginBase::UIntSet::const_iterator it=removedCP2.begin();it!=removedCP2.end();++it)
             {
@@ -693,7 +799,7 @@ PanoCommand::PanoCommand* CleanControlPointsOperation::GetInternalCommand(wxWind
     {
         return NULL;
     }
-    if (removedCPs.size()>0)
+    if (!removedCPs.empty())
     {
         wxMessageBox(wxString::Format(_("Removed %lu control points"), (unsigned long int)removedCPs.size()), _("Cleaning"), wxOK | wxICON_INFORMATION, parent);
         return new PanoCommand::RemoveCtrlPointsCmd(pano,removedCPs);
@@ -733,7 +839,7 @@ PanoCommand::PanoCommand* CelesteOperation::GetInternalCommand(wxWindow* parent,
     {
         // Image to analyse
         HuginBase::CPointVector cps=pano.getCtrlPointsVectorForImage(*it);
-        if(cps.size()==0)
+        if(cps.empty())
         {
             if (!progress.updateDisplayValue())
             {
@@ -768,7 +874,7 @@ PanoCommand::PanoCommand* CelesteOperation::GetInternalCommand(wxWindow* parent,
         {
             return NULL;
         };
-        if(cloudCP.size()>0)
+        if(!cloudCP.empty())
         {
             for(HuginBase::UIntSet::const_iterator it2=cloudCP.begin();it2!=cloudCP.end(); ++it2)
             {
@@ -785,7 +891,7 @@ PanoCommand::PanoCommand* CelesteOperation::GetInternalCommand(wxWindow* parent,
     {
         return NULL;
     }
-    if (cpsToRemove.size()>0)
+    if (!cpsToRemove.empty())
     {
         wxMessageBox(wxString::Format(_("Removed %lu control points"), (unsigned long int) cpsToRemove.size()), _("Celeste result"),wxOK|wxICON_INFORMATION);
         return new PanoCommand::RemoveCtrlPointsCmd(pano,cpsToRemove);
@@ -862,14 +968,10 @@ PanoCommand::PanoCommand* ResetOperation::GetInternalCommand(wxWindow* parent, H
             return NULL;
         };
     };
-    if(images.size()==0)
+    if(images.empty())
     {
         fill_set(images,0,pano.getNrOfImages()-1);
     };
-    // If we should unlink exposure value (to load it from EXIF)
-    bool needs_unlink_exposure = false;
-    bool needs_unlink_redbal = false;
-    bool needs_unlink_bluebal = false;
     double redBalanceAnchor = pano.getImage(pano.getOptions().colorReferenceImage).getExifRedBalance();
     double blueBalanceAnchor = pano.getImage(pano.getOptions().colorReferenceImage).getExifBlueBalance();
     if(fabs(redBalanceAnchor)<1e-2)
@@ -933,12 +1035,6 @@ PanoCommand::PanoCommand* ResetOperation::GetInternalCommand(wxWindow* parent, H
         {
             if(m_resetExposure==1 || m_resetExposure==3)
             {
-                if (pano.getImage(imgNr).ExposureValueisLinked())
-                {
-                    /* Unlink exposure value variable so the EXIF values can be
-                     * independant. */
-                    needs_unlink_exposure = true;
-                }
                 //reset to exif value
                 double eV=srcImg.calcExifExposureValue();
                 if ((m_resetExposure == 1 && eV != 0) || m_resetExposure == 3)
@@ -956,18 +1052,6 @@ PanoCommand::PanoCommand* ResetOperation::GetInternalCommand(wxWindow* parent, H
         {
             if(m_resetColor==1)
             {
-                if (pano.getImage(imgNr).WhiteBalanceRedisLinked())
-                {
-                    /* Unlink red balance variable so the EXIF values can be
-                     * independant. */
-                    needs_unlink_redbal = true;
-                }
-                if (pano.getImage(imgNr).WhiteBalanceBlueisLinked())
-                {
-                    /* Unlink red balance variable so the EXIF values can be
-                     * independant. */
-                    needs_unlink_bluebal = true;
-                }
                 double redBal=1;
                 double blueBal=1;
                 const HuginBase::SrcPanoImage& img=pano.getImage(imgNr);
@@ -1016,48 +1100,6 @@ PanoCommand::PanoCommand* ResetOperation::GetInternalCommand(wxWindow* parent, H
         vars.push_back(ImgVars);
     };
     std::vector<PanoCommand::PanoCommand *> reset_commands;
-    if (needs_unlink_exposure)
-    {
-        std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> variables;
-        variables.insert(HuginBase::ImageVariableGroup::IVE_ExposureValue);
-        
-        reset_commands.push_back(
-                new PanoCommand::ChangePartImagesLinkingCmd(
-                            pano,
-                            images,
-                            variables,
-                            false,
-                            HuginBase::StandardImageVariableGroups::getLensVariables())
-                );
-    }
-    if (needs_unlink_redbal)
-    {
-        std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> variables;
-        variables.insert(HuginBase::ImageVariableGroup::IVE_WhiteBalanceRed);
-        
-        reset_commands.push_back(
-                new PanoCommand::ChangePartImagesLinkingCmd(
-                            pano,
-                            images,
-                            variables,
-                            false,
-                            HuginBase::StandardImageVariableGroups::getLensVariables())
-                );
-    }
-    if (needs_unlink_bluebal)
-    {
-        std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> variables;
-        variables.insert(HuginBase::ImageVariableGroup::IVE_WhiteBalanceBlue);
-        
-        reset_commands.push_back(
-                new PanoCommand::ChangePartImagesLinkingCmd(
-                            pano,
-                            images,
-                            variables,
-                            false,
-                            HuginBase::StandardImageVariableGroups::getLensVariables())
-                );
-    }
     reset_commands.push_back(
                             new PanoCommand::UpdateImagesVariablesCmd(pano, images, vars)
                                            );
@@ -1146,7 +1188,7 @@ bool ResetOperation::ShowDialog(wxWindow* parent)
 
 bool NewStackOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
-    if(pano.getNrOfImages()==0 || images.size()==0)
+    if(pano.getNrOfImages()==0 || images.empty())
     {
         return false;
     }
@@ -1169,13 +1211,13 @@ PanoCommand::PanoCommand* NewStackOperation::GetInternalCommand(wxWindow* parent
 
 bool ChangeStackOperation::IsEnabled(HuginBase::Panorama& pano, HuginBase::UIntSet images, GuiLevel guiLevel)
 {
-    if(pano.getNrOfImages()==0 || images.size()==0)
+    if(pano.getNrOfImages()==0 || images.empty())
     {
         return false;
     }
     else
     {
-        //project must have more than 1 stack before you can assign an other stack number
+        //project must have more than 1 stack before you can assign another stack number
         HuginBase::StandardImageVariableGroups variableGroups(pano);
         return variableGroups.getStacks().getNumberOfParts() > 1;
     };
@@ -1328,6 +1370,7 @@ void GeneratePanoOperationVector()
     PanoOpImages.push_back(new RemoveImageOperation());
     PanoOpImages.push_back(new ChangeAnchorImageOperation());
     PanoOpImages.push_back(new ChangeColorAnchorImageOperation());
+    PanoOpImages.push_back(new ImageVariablesExpressionOperation());
 
     PanoOpLens.push_back(new NewLensOperation());
     PanoOpLens.push_back(new ChangeLensOperation());
