@@ -122,7 +122,7 @@ void PhotometricOptimizer::OptimData::ToX(double * x)
 {
     for (size_t i=0; i < m_vars.size(); i++)
     {
-        assert(m_vars[i].imgs.size() > 0);
+        assert(!m_vars[i].imgs.empty());
             // get corresponding image number
         unsigned j = *(m_vars[i].imgs.begin());
             // get value
@@ -137,7 +137,7 @@ void PhotometricOptimizer::OptimData::FromX(double * x)
     for (size_t i=0; i < m_vars.size(); i++)
     {
         // TODO: transform some variables, such as the vignetting center!
-        assert(m_vars[i].imgs.size() > 0);
+        assert(!m_vars[i].imgs.empty());
             // copy value int all images
         for (std::set<unsigned>::const_iterator it = m_vars[i].imgs.begin();
              it != m_vars[i].imgs.end(); ++it)
@@ -277,6 +277,7 @@ int PhotometricOptimizer::photometricVis(double *p, double *x, int m, int n, int
 
 void PhotometricOptimizer::optimizePhotometric(PanoramaData & pano, const OptimizeVector & vars,
                                                const std::vector<vigra_ext::PointPairRGB> & correspondences,
+                                               const float imageStepSize,
                                                AppBase::ProgressDisplay* progress,
                                                double & error)
 {
@@ -304,7 +305,7 @@ void PhotometricOptimizer::optimizePhotometric(PanoramaData & pano, const Optimi
     };
 
     int nMaxIter = 250;
-    OptimData data(pano, photometricVars, correspondences, 5/255.0, false, nMaxIter, progress);
+    OptimData data(pano, photometricVars, correspondences, 5 * imageStepSize, false, nMaxIter, progress);
 
     double info[LM_INFO_SZ];
 
@@ -327,15 +328,16 @@ void PhotometricOptimizer::optimizePhotometric(PanoramaData & pano, const Optimi
     // TODO: setup optimisation options with some good defaults.
     double optimOpts[5];
     
-    optimOpts[0] = 1E-03;  // init mu
+    optimOpts[0] = LM_INIT_MU;  // init mu
     // stop thresholds
-    optimOpts[1] = 1e-5;   // ||J^T e||_inf
-    optimOpts[2] = 1e-5;   // ||Dp||_2
-    optimOpts[3] = 1e-1;   // ||e||_2
+    optimOpts[1] = LM_STOP_THRESH;   // ||J^T e||_inf
+    optimOpts[2] = LM_STOP_THRESH;   // ||Dp||_2
+    optimOpts[3] = std::pow(imageStepSize*0.1f, 2);   // ||e||_2
     // difference mode
     optimOpts[4] = LM_DIFF_DELTA;
     
     dlevmar_dif(&photometricError, &photometricVis, &(p[0]), &(x[0]), m, n, nMaxIter, optimOpts, info, NULL,NULL, &data);  // no jacobian
+
     // copy to source images (data.m_imgs)
     data.FromX(p.begin());
     // calculate error at solution
@@ -369,8 +371,15 @@ bool IsHighVignetting(std::vector<double> vigCorr)
     srcImage.setRadialVigCorrCoeff(vigCorr);
     srcImage.setSize(vigra::Size2D(500, 500));
     Photometric::ResponseTransform<double> transform(srcImage);
-    transform.enforceMonotonicity();
-    return transform(1.0, hugin_utils::FDiff2D(0,0))<0.7;
+    for (size_t x = 0; x < 250; x += 10)
+    {
+        const double vigFactor = transform.calcVigFactor(hugin_utils::FDiff2D(x, x));
+        if (vigFactor<0.7 || vigFactor>1.1)
+        {
+            return true;
+        };
+    };
+    return false;
 };
 
 bool CheckStrangeWB(PanoramaData & pano)
@@ -391,6 +400,7 @@ bool CheckStrangeWB(PanoramaData & pano)
 
 void SmartPhotometricOptimizer::smartOptimizePhotometric(PanoramaData & pano, PhotometricOptimizeMode mode,
                                                           const std::vector<vigra_ext::PointPairRGB> & correspondences,
+                                                          const float imageStepSize, 
                                                           AppBase::ProgressDisplay* progress,
                                                           double & error)
 {
@@ -407,7 +417,7 @@ void SmartPhotometricOptimizer::smartOptimizePhotometric(PanoramaData & pano, Ph
         vars = OPT_EXP;
         optimizePhotometric(pano, 
                             createOptVars(pano, vars, opts.colorReferenceImage),
-                            correspondences, progress, error);
+                            correspondences, imageStepSize, progress, error);
     };
 
     if(!singleStack)
@@ -418,7 +428,7 @@ void SmartPhotometricOptimizer::smartOptimizePhotometric(PanoramaData & pano, Ph
         VariableMapVector oldVars = pano.getVariables();
         optimizePhotometric(pano, 
                             createOptVars(pano, vars, opts.colorReferenceImage),
-                            correspondences, progress, error);
+                            correspondences, imageStepSize, progress, error);
         // check if vignetting is plausible
         if(IsHighVignetting(pano.getImage(0).getRadialVigCorrCoeff()))
         {
@@ -437,7 +447,7 @@ void SmartPhotometricOptimizer::smartOptimizePhotometric(PanoramaData & pano, Ph
     VariableMapVector oldVars = pano.getVariables();
     optimizePhotometric(pano, 
                         createOptVars(pano, vars, opts.colorReferenceImage),
-                        correspondences, progress, error);
+                        correspondences, imageStepSize, progress, error);
     // now check the results
     const bool hasHighVignetting = IsHighVignetting(pano.getImage(0).getRadialVigCorrCoeff());
     // @TODO check also response curve, what parameters are considered invalid?
@@ -458,7 +468,7 @@ void SmartPhotometricOptimizer::smartOptimizePhotometric(PanoramaData & pano, Ph
         {
             optimizePhotometric(pano, 
                                 createOptVars(pano, vars, opts.colorReferenceImage),
-                                correspondences, progress, error);
+                                correspondences, imageStepSize, progress, error);
         };
     };
 }
@@ -467,7 +477,7 @@ void SmartPhotometricOptimizer::smartOptimizePhotometric(PanoramaData & pano, Ph
 bool PhotometricOptimizer::runAlgorithm()
 {
     optimizePhotometric(o_panorama, 
-                        o_vars, o_correspondences,
+                        o_vars, o_correspondences, o_imageStepSize,
                         getProgressDisplay(), o_resultError);
     
     // optimizePhotometric does not tell us if it's cancelled
@@ -483,7 +493,7 @@ bool SmartPhotometricOptimizer::runAlgorithm()
 {
     smartOptimizePhotometric(o_panorama,
                              o_optMode,
-                             o_correspondences,
+                             o_correspondences, o_imageStepSize,
                              getProgressDisplay(), o_resultError);
     
     // smartOptimizePhotometric does not tell us if it's cancelled
